/****************************************************
  Projekt: TMCM-ARM (TMCL fr ARM-Module)

  Modul:   ADCIrq.c
           ADC Interrupt Handler

  Datum:   26.4.2007 OK
*****************************************************/


#include "at91sam7x256.h"
#include "bits.h"

#include "Eval429.h"

//Array fr die Analogeingnge
static volatile UINT AnalogIn[8];


/*******************************************************************
  ADC-Interrupthandler
  Wird durch den AIC aufgerufen, wenn eine A/D-Wandlung fertig ist.
********************************************************************/
void ADCInterrupt(void)
{
  UINT StatusRegister;

  //Interruptflags lesen (und damit auch zurcksetzen)
  StatusRegister=AT91C_BASE_ADC->ADC_SR;

  //Kanal einlesen, dessen Wandlung fertig ist
  if(StatusRegister & AT91C_ADC_EOC0) AnalogIn[0]=AT91C_BASE_ADC->ADC_CDR0;
  if(StatusRegister & AT91C_ADC_EOC1) AnalogIn[1]=AT91C_BASE_ADC->ADC_CDR1;
  if(StatusRegister & AT91C_ADC_EOC2) AnalogIn[2]=AT91C_BASE_ADC->ADC_CDR2;
  if(StatusRegister & AT91C_ADC_EOC3) AnalogIn[3]=AT91C_BASE_ADC->ADC_CDR3;
  if(StatusRegister & AT91C_ADC_EOC4) AnalogIn[4]=AT91C_BASE_ADC->ADC_CDR4;
  if(StatusRegister & AT91C_ADC_EOC5) AnalogIn[5]=AT91C_BASE_ADC->ADC_CDR5;
  if(StatusRegister & AT91C_ADC_EOC6) AnalogIn[6]=AT91C_BASE_ADC->ADC_CDR6;
  if(StatusRegister & AT91C_ADC_EOC7)
  {
    AnalogIn[7]=AT91C_BASE_ADC->ADC_CDR7;

    //Das war der letzte Kanal, also Wandlung neu starten
    AT91C_BASE_ADC->ADC_CR=AT91C_ADC_START;
  }
}


/*******************************************************************
   Funktion: InitADC()
   Parameter: ---

   Rckgabewert: ---

   Zweck: Initialisierung des ADC
********************************************************************/
void InitADC(void)
{
  //ADC konfigurieren und alle Kanle aktivieren
  AT91C_BASE_ADC->ADC_MR=AT91C_ADC_TRGEN_DIS|AT91C_ADC_LOWRES_10_BIT|AT91C_ADC_SLEEP_NORMAL_MODE|
                         (18<<8)|(4<<16)|(7<<24);  //Prescaler=2 (=> 8Mhz), Startup Time=4, Sample&Hold Time=7
  AT91C_BASE_ADC->ADC_CHER=AT91C_ADC_CH0|AT91C_ADC_CH1|AT91C_ADC_CH2|AT91C_ADC_CH3|
                           AT91C_ADC_CH4|AT91C_ADC_CH5|AT91C_ADC_CH6|AT91C_ADC_CH7;

  //Interruptfunktion fr den ADC setzen (im AIC)
  AT91C_BASE_AIC->AIC_IDCR=1<<AT91C_ID_ADC;
  AT91C_BASE_AIC->AIC_SVR[AT91C_ID_ADC]=(unsigned int) ADCInterrupt;
  AT91C_BASE_AIC->AIC_SMR[AT91C_ID_ADC]=ADC_INTERRUPT_PRIORITY|AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL;
  AT91C_BASE_AIC->AIC_ICCR=1<<AT91C_ID_ADC;

  //Interrupt nur fr den letzten Kanal akivieren (steigert die Performanz ungemein!)
  AT91C_BASE_ADC->ADC_IER = AT91C_ADC_CH7;
  AT91C_BASE_ADC->ADC_IDR = AT91C_ADC_CH6|AT91C_ADC_CH5|AT91C_ADC_CH4|AT91C_ADC_CH3|AT91C_ADC_CH2|AT91C_ADC_CH1|AT91C_ADC_CH0;

  //Interrupt fr ADC im AIC freischalten
  AT91C_BASE_AIC->AIC_IECR = 1<<AT91C_ID_ADC;

  //Erste Konvertierung starten
  AT91C_BASE_ADC->ADC_CR=AT91C_ADC_START;
}


UINT GetADCValue(UINT Index)
{
  return AnalogIn[Index];
}
