/*******************************************************
  Projekt: TMC429-TMC26x-EVAL

  Modul:   Eval429-ExtFlash.c
           Zugriff auf das externe Flash-ROM
           (AMIC A25L016M-F)

  Datum:   16.3.2009 OK
********************************************************/


#include "at91sam7x256.h"
#include "bits.h"

#include "Eval429.h"
#include "Eval429-SPI.h"

//Hier Type des FlashROMs definieren! Es sind eigentlich nur
//die Lschbefehle anders!

//#define FLASH_TYPE_AT25F2048
#define FLASH_TYPE_M25P80   //geht auch fr A25L016M-F


/*******************************************************************
   Funktion: WriteExtFlashByte
   Parameter: Address: Adresse im Flash (0..3FFFF)
              Value: der zu schreibende Wert

   Rckgabewert: ---

   Zweck: Schreiben eines Bytes in das externe Flash. Der entsprechende
   Sektor (oder der ganze Chip) mu vorher gelscht worden sein!
********************************************************************/
void WriteExtFlashByte(UINT Address, UCHAR Value)
{
  //Schreiben erlauben
  ReadWriteSPI(SPI_DEV_FLASH, 0x06, TRUE);  //Befehl "Write Enable"
  do
  {
    ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
  } while((ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x02)==0x00);  //Warte bis "Write Enable"-Bit gesetzt

  //Eigentliches Schreiben
  ReadWriteSPI(SPI_DEV_FLASH, 0x02, FALSE); //Befehl "Write"
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 16, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 8, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address & 0xff, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Value, TRUE);

  //Warten bis Schreibvorgang beendet ist
  do
  {
    ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
  } while(ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x01);
}


/*******************************************************************
   Funktion: WriteExtFlashBlock
   Parameter: Address: Adresse im EEPROM (0..3FFFF)
              Block: Startadresse des zu schreibenden Blocks
              Size: Lnge des Blocks in Bytes

   Rckgabewert: ---

   Zweck: Schreiben mehrerer Bytes in das externe Flash.
   Dabei knnen beliebig viele Bytes (also auch das gesamte Flash
   beschrieben werden (die speziellen Eigenschaften des 25F2048 bzw.
   M25P80 werden dabei beachtet).
   Der entsprechende Sektor bzw. der ganze Chip mu vorher gelscht
   worden sein!
********************************************************************/
void WriteExtFlashBlock(UINT Address, UCHAR *Block, UINT Size)
{
  UINT i;

  //Schreiben erlauben
  ReadWriteSPI(SPI_DEV_FLASH, 0x06, TRUE);  //Befehl "Write Enable"
  do
  {
    ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
  } while((ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x02)==0x00);  //Warte bis "Write Enable"-Bit gesetzt

  //Schreibvorgang (Startadresse)
  ReadWriteSPI(SPI_DEV_FLASH, 0x02, FALSE); //Befehl "Write"
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 16, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 8, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address & 0xff, FALSE);

  //Eigentliches Schreiben der Daten
  for(i=0; i<Size; i++)
  {
    //Adresse mitzhlen und bei berschreiten einer Seitengrenze das Flash deselektieren
    //und neuen Write-Befehl senden (bzw. beim letzten Datenbyte einfach nur Flash
    //deselektieren).
    //Dies ist erforderlich, da beim Beschreiben im 25F2048 nicht ber eine Seitengrenze
    //hinaus geschrieben werden kann.
    Address++;
    ReadWriteSPI(SPI_DEV_FLASH, *(Block+i), (Address & 0x000000ff)==0 || i==Size-1);
    if((Address & 0x000000ff)==0 && i<Size-1)  //ber Seitengrenze hinaus, aber noch Bytes zu schreiben?
    {
      //Warte bis Schreibvorgang beendet
      do
      {
        ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
      } while(ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x01);

      //Neuer "Write Enable"-Befehl
      ReadWriteSPI(SPI_DEV_FLASH, 0x06, TRUE);  //Befehl "Write Enable"
      do
      {
        ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
      } while((ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x02)==0x00);  //Warte bis "Write Enable"-Bit gesetzt

      //Neuer "Write"-Befehl (mit der nchsten Adresse)
      ReadWriteSPI(SPI_DEV_FLASH, 0x02, FALSE); //Befehl "Write"
      ReadWriteSPI(SPI_DEV_FLASH, Address >> 16, FALSE);
      ReadWriteSPI(SPI_DEV_FLASH, Address >> 8, FALSE);
      ReadWriteSPI(SPI_DEV_FLASH, Address & 0xff, FALSE);
    }
  }

  //Warte bis Schreibvorgang beendet
  do
  {
    ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
  } while(ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x01);
}


/*******************************************************************
   Funktion: EraseExtFlashSector
   Parameter: Address: irgendeine Adresse im zu lschenden Sektor
              Wait: TRUE wenn auf Ende des Lschvorgangs gewartet
                         werden soll
                    FALSE wenn nicht gewartet werden soll

   Rckgabewert: ---

   Zweck: Lschen eines (64 KByte groen) Sektors im Flash.
   Dies dauert bis zu 3 Sekunden!
********************************************************************/
void EraseExtFlashSector(UINT Address, UINT Wait)
{
  //Schreiben erlauben
  ReadWriteSPI(SPI_DEV_FLASH, 0x06, TRUE);  //Befehl "Write Enable"
  do
  {
    ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
  } while((ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x02)==0x00);  //Warte bis "Write Enable"-Bit gesetzt

  //Lschvorgang starten
#if defined(FLASH_TYPE_AT25F2048)
  ReadWriteSPI(SPI_DEV_FLASH, 0x52, FALSE); //Befehl "Sector Erase"
#elif defined(FLASH_TYPE_M25P80)
  ReadWriteSPI(SPI_DEV_FLASH, 0xd8, FALSE); //Befehl "Sector Erase"
#else
#error "Flash type not defined!"
#endif
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 16, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 8, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address & 0xff, TRUE);

  //Warten bis Lschvorgang beendet
  if(Wait)
    do
    {
      AT91C_BASE_WDTC->WDTC_WDCR=(0xA5 << 24)|AT91C_WDTC_WDRSTT;  //Watchdog-Reset
      ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
    } while(ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x01);
}


/*******************************************************************
   Funktion: EraseExtFlash
   Parameter: Wait: TRUE wenn auf Ende des Lschvorgangs gewartet
                         werden soll
                    FALSE wenn nicht gewartet werden soll

   Rckgabewert: ---

   Zweck: Lschen deas gesamten externen Flash. Falls die Schutzbits
   gesetzt sind, dann werden diese vorher gelscht.

   Dies dauert bis zu 4 Sekunden beim AT25F2048 bzw. bis zu 20
   Sekunden (typisch 10 Sekunden) beim M25P80!
********************************************************************/
void EraseExtFlash(UINT Wait)
{
  UCHAR Status;

  //Schreiben erlauben
  ReadWriteSPI(SPI_DEV_FLASH, 0x06, TRUE);  //Befehl "Write Enable"
  do
  {
    ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
    Status=ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE);
  } while((Status & 0x02)==0x00);  //Warte bis "Write Enable"-Bit gesetzt

  //Protection Bits gesetzt?
  if(Status & 0x1c)  //BP2, BP1 oder BP0 gesetzt?
  {
    ReadWriteSPI(SPI_DEV_FLASH, 0x01, FALSE);  //Befehl "Write Status Register"
    ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE); //Alle Schutzbits lschen

    //Warten bis Vorgang beendet
    do
    {
      ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
    } while(ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x01);

    //und nun nochmal Write Enable
    ReadWriteSPI(SPI_DEV_FLASH, 0x06, TRUE);  //Befehl "Write Enable"
    do
    {
      ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
    } while((ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x02)==0x00);  //Warte bis "Write Enable"-Bit gesetzt
  }

  //Lschvorgang starten
#if defined(FLASH_TYPE_AT25F2048)
  ReadWriteSPI(SPI_DEV_FLASH, 0x62, TRUE); //Befehl "Chip Erase"
#elif defined(FLASH_TYPE_M25P80)
  ReadWriteSPI(SPI_DEV_FLASH, 0xc7, TRUE); //Befehl "Chip Erase"
#else
#error "Flash type not defined!"
#endif

  //Warten bis Lschvorgang beendet
  if(Wait)
    do
    {
      AT91C_BASE_WDTC->WDTC_WDCR=(0xA5 << 24)|AT91C_WDTC_WDRSTT;  //Watchdog-Reset
      ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
    } while(ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x01);
}

/*******************************************************************
   Funktion: ReadExtFlashByte
   Parameter: Address: Adresse im externen Flash (0..3FFFF)

   Rckgabewert: der gelesene Wert

   Zweck: Lesen eines Bytes aus dem externen Flash.
********************************************************************/
UCHAR ReadExtFlashByte(UINT Address)
{
#if defined(FLASH_TYPE_AT25F2048)
  ReadWriteSPI(SPI_DEV_FLASH, 0x03, FALSE);  //Befehl "Read"
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 16, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 8, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address & 0xff, FALSE);
#elif defined(FLASH_TYPE_M25P80)
  ReadWriteSPI(SPI_DEV_FLASH, 0x0b, FALSE);  //Befehl "Fast Read" (fr SPI-Frequenz bis 25MHz)
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 16, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 8, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address & 0xff, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, 0x00, FALSE);  //bentigt ein Dummy-Byte
#else
#error "Flash type not defined!"
#endif

  return ReadWriteSPI(SPI_DEV_FLASH, 0, TRUE);
}


/*******************************************************************
   Funktion: ReadExtFlashBlock
   Parameter: Address: Adresse im EEPROM (0..3FFFF)
              Block: Startadresse des zu lesenden Blocks
              Size: Lnge des Blocks in Bytes

   Rckgabewert: ---

   Zweck: Lesen mehrerer Bytes aus dem externen Flash.
   Dabei drfen ab beliebiger Adresse beliebig viele Bytes gelesen
   werden.
********************************************************************/
void ReadExtFlashBlock(UINT Address, UCHAR *Block, UINT Size)
{
  UINT i;

#if defined(FLASH_TYPE_AT25F2048)
  ReadWriteSPI(SPI_DEV_FLASH, 0x03, FALSE);  //Befehl "Read"
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 16, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 8, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address & 0xff, FALSE);
#elif defined(FLASH_TYPE_M25P80)
  ReadWriteSPI(SPI_DEV_FLASH, 0x0b, FALSE);  //Befehl "Fast Read" (fr SPI-Frequenz bis 25MHz)
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 16, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address >> 8, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, Address & 0xff, FALSE);
  ReadWriteSPI(SPI_DEV_FLASH, 0x00, FALSE);  //bentigt ein Dummy-Byte
#else
#error "Flash type not defined!"
#endif

  for(i=0; i<Size; i++)
    *(Block+i)=ReadWriteSPI(SPI_DEV_FLASH, 0, i==Size-1);  //beim letzten Byte Flash deselektieren
}


/*******************************************************************
   Funktion: GetExtFlashReady
   Parameter: ---

   Rckgabewert: FALSE wenn kein Schreib- oder Lschvorgang in Arbeit
                 TRUE wenn Schreib- oder Lschvorgang in Arbeit

   Zweck: Abfrage, ob gerade ein Schreib- oder Lschvorgang im Flash
   ausgefhrt wird. Dies ist insbesondere nach dem Lschen mit
   EraseExtFlash() oder EraseExtFlashSector() notwendig.
********************************************************************/
UCHAR GetExtFlashBusy(void)
{
  ReadWriteSPI(SPI_DEV_FLASH, 0x05, FALSE);  //Befehl "Get Status"
  return ReadWriteSPI(SPI_DEV_FLASH, 0x00, TRUE) & 0x01;
}
