/****************************************************
  Projekt: TMCM-ARM (TMCL fr ARM-Module)

  Modul:   TMCM-ARM.c
           Hauptprogramm fr die TMCM-Module mit ARM.

  Datum:   23.3.2007 OK
*****************************************************/


#include <stdlib.h>
#include "at91sam7x256.h"
#include "bits.h"

#include "Eval429.h"
#include "Eval429-UART.h"
#include "Eval429-SPI.h"
#include "Eval429-Systimer.h"
#include "Eval429-TMCL.h"
#include "Eval429-TMC262.h"
#include "Eval429-TMC428.h"
#include "Eval429-USB.h"
#include "Eval429-ADCIrq.h"
#include "Eval429-ExtFlash.h"

const char *VersionString="429V2.04";

static UINT ActualAxis;
static UINT DebounceLeft;
static UINT DebounceRight;
static UINT DebounceCool;
static UINT HardStopDebounce;
static UINT ExtSPIMode;

UCHAR DriverFlags[N_O_MOTORS];
UINT StallLevel[N_O_MOTORS];
UCHAR SmartEnergy[N_O_MOTORS];
volatile UINT Dummy;


/*******************************************************************
   Funktion: InitIO
   Parameter: ---

   Rckgabewert: ---
   Zweck: Initialisierung aller Portpins der CPU.
********************************************************************/
void InitIO(void)
{
	UINT Count1;
	UINT Count2;
	UINT i;
	
  RESET_WATCHDOG();

  //Clock-Ausgang fr den TMC262 einschalten mit Vorteiler (Main Clock/1 (16Mhz))
  AT91C_BASE_PIOB->PIO_PDR=BIT22|BIT21|BIT0;
  AT91C_BASE_PIOB->PIO_BSR=BIT22|BIT21|BIT0;
  AT91C_BASE_PMC->PMC_SCER=AT91C_PMC_PCK2|AT91C_PMC_PCK1|AT91C_PMC_PCK0;
  AT91C_BASE_PMC->PMC_PCKR[0]=AT91C_PMC_PRES_CLK|AT91C_PMC_CSS_MAIN_CLK;   //16Mhz an PCK1 (PB0)
  AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK|AT91C_PMC_CSS_MAIN_CLK;   //16Mhz an PCK1 (PB21)
  //AT91C_BASE_PMC->PMC_PCKR[1]=AT91C_PMC_PRES_CLK_2|AT91C_PMC_CSS_PLL_CLK;   //48Mhz an PCK1 (PB21)
  AT91C_BASE_PMC->PMC_PCKR[2]=AT91C_PMC_PRES_CLK|AT91C_PMC_CSS_MAIN_CLK;   //16Mhz an PCK2 (PB22)

  //Externen Reset ermglichen
  AT91C_BASE_RSTC->RSTC_RMR = 0xA5000001;

  //Clock fr PIOA und PIOB einschalten
  AT91C_BASE_PMC->PMC_PCER=1 << AT91C_ID_PIOA;
  AT91C_BASE_PMC->PMC_PCER=1 << AT91C_ID_PIOB;

  //Initialisierung der PIO A
  AT91C_BASE_PIOA->PIO_PER=BIT30|BIT29|BIT28|BIT27|BIT26|BIT25|BIT20|BIT19|BIT11|BIT10|BIT9|BIT8|
                           BIT7|BIT6|BIT5|BIT4|BIT3|BIT2;
  AT91C_BASE_PIOA->PIO_OER=BIT30|BIT28|BIT29|BIT25|BIT11|BIT10|BIT9|BIT8|BIT6|BIT1;
  AT91C_BASE_PIOA->PIO_ODR=BIT27|BIT26|BIT20|BIT19|BIT7|BIT5|BIT4|BIT3|BIT2;
  AT91C_BASE_PIOA->PIO_CODR=BIT30|BIT29|BIT10|BIT9|BIT8|BIT6;
  AT91C_BASE_PIOA->PIO_SODR=BIT25|BIT11;
  AT91C_BASE_PIOA->PIO_PPUDR=BIT5;

  //Initialisierung der PIO B
  AT91C_BASE_PIOB->PIO_PER=BIT30|BIT29|BIT28|BIT27|BIT26|BIT25|BIT24|BIT23|BIT20|BIT19|BIT18|BIT17|BIT16|BIT15|
                           BIT14|BIT13|BIT12|BIT11|BIT10|BIT9|BIT8|BIT7|BIT6|BIT5|BIT4|BIT3|BIT2|BIT1;
  AT91C_BASE_PIOB->PIO_OER=BIT25|BIT24|BIT20|BIT13|BIT5|BIT4|BIT3;
  AT91C_BASE_PIOB->PIO_ODR=BIT30|BIT29|BIT28|BIT27|BIT26|BIT23|BIT19|BIT18|BIT17|BIT16|BIT15|
                           BIT14|BIT12|BIT11|BIT10|BIT9|BIT8|BIT7|BIT6|BIT2|BIT1;
  AT91C_BASE_PIOB->PIO_SODR=BIT13;
  AT91C_BASE_PIOB->PIO_SODR=BIT5|BIT4|BIT2;
  AT91C_BASE_PIOB->PIO_PPUDR=BIT30|BIT29|BIT28|BIT27;

  //PWM0 aktivieren fr analoge Ausgabe der stallGuard2-Werte
  //Clock fr PWM Controller einschalten
  AT91C_BASE_PMC->PMC_PCER=1 << AT91C_ID_PWMC;

  //PWM-Signale auf die entsprechenden Pins schalten
  AT91C_BASE_PIOB->PIO_PDR=BIT19;
  AT91C_BASE_PIOB->PIO_ASR=BIT19;

  //PWM 0 einstellen
  AT91C_BASE_PWMC_CH0->PWMC_CPRDR=1024;               //46875Hz
  AT91C_BASE_PWMC_CH0->PWMC_CMR=AT91C_PWMC_CPOL;      //Clk/1, normale PWM, Duty Cycle bei nchster Periode bernehmen
  AT91C_BASE_PWMC_CH0->PWMC_CDTYR=1;                  //Tastverhltnis erst einmal 0%

  //PWM 0 einschalten
  AT91C_BASE_PWMC->PWMC_ENA=AT91C_PWMC_CHID0;

  //Reset des TMC424 wieder aufheben
  AT91C_BASE_PIOA->PIO_SODR=BIT29;

  //Prfen, ob die Pins RxD und TxD der RS232-Schnittstelle miteinander verbunden sind.
  //Wenn ja: Modus fr direkten Betrieb der ICs ber SPI einschalten.
  Count1=Count2=0;
  for(i=0; i<10; i++)
  {
    AT91C_BASE_PIOA->PIO_CODR=BIT28;
    for(Dummy=0; Dummy<1000; Dummy++);
    if(!(AT91C_BASE_PIOA->PIO_PDSR & BIT27)) Count1++;

    AT91C_BASE_PIOA->PIO_SODR=BIT28;
    for(Dummy=0; Dummy<1000; Dummy++);
    if((AT91C_BASE_PIOA->PIO_PDSR & BIT27)) Count2++;
  }
  if(Count1==10 && Count2==10)
  {
    ExtSPIMode=TRUE;
    AT91C_BASE_PIOA->PIO_ODR=BIT25;
  }
  else ExtSPIMode=FALSE;
}


/*******************************************************************
   Funktion: SetPWMOutpuut
   Parameter: PWMValue: PWM-Tastverhltnis (0..1024 => 0..100%)

   Rckgabewert: ---
   Zweck: Setzen des Tastverhltnisses des PWM-Ausgangs fr analoge
          Megerte.
********************************************************************/
void SetPWMOutput(UINT PWMValue)
{
  if(PWMValue==0)
    AT91C_BASE_PWMC_CH0->PWMC_CDTYR=1;  //PWM-Wert nicht auf Null setzen (Fehler beim AT91SAM7X256)
  else
    AT91C_BASE_PWMC_CH0->PWMC_CDTYR=PWMValue;
}


/*******************************************************************
   Funktion: CheckDrivers
   Parameter: ---

   Rckgabewert: ---
   Zweck: Auslesen der Zustnde der Motortreiber.
********************************************************************/
void CheckDrivers(void)
{
  if(Get262DriverReadSelect(ActualAxis)!=TMC262_RB_SMART_ENERGY)
    Set262DriverReadSelect(ActualAxis, TMC262_RB_SMART_ENERGY);

  Read262State(ActualAxis, NULL, NULL, &StallLevel[ActualAxis], &SmartEnergy[ActualAxis], &DriverFlags[ActualAxis]);
  if(ActualAxis==0) SetPWMOutput(StallLevel[0]);

  ActualAxis++;
  if(ActualAxis>=N_O_MOTORS) ActualAxis=0;
}


/*******************************************************************
   Funktion: ProcessButtons
   Parameter: ---

   Rckgabewert: ---
   Zweck: Abfrage der Taster und entprechende Reaktion.
********************************************************************/
void ProcessButtons(void)
{
  int i;

  //Tasten "LEFT" und "RIGHT"
  if(BUTTON_LEFT() && BUTTON_RIGHT())
  {
    Write428Short(IDX_VTARGET|MOTOR0, 0);
    Write428Short(IDX_VACTUAL|MOTOR0, 0);
    HardStopDebounce=BUTTON_DEBOUNCE_MAX;
  }
  else if(HardStopDebounce>0)
  {
    if(!BUTTON_LEFT() && !BUTTON_RIGHT()) HardStopDebounce--;
  }
  else
  {
    if(BUTTON_LEFT())
    {
      if(DebounceLeft<BUTTON_DEBOUNCE_MAX) DebounceLeft++;
      else if(DebounceLeft==BUTTON_DEBOUNCE_MAX)
      {
        Set428RampMode(0, RM_VELOCITY);
        Write428Short(IDX_VMAX|MOTOR0, 2047);
        DebounceLeft=0;

        i=Read428Short(IDX_VACTUAL|MOTOR0)-1;
        if(i<-2047) i=-2047;
        Write428Short(IDX_VTARGET|MOTOR0, i);
        Write428Short(IDX_VACTUAL|MOTOR0, i);
      }
    }
    else DebounceLeft=0;

    if(BUTTON_RIGHT())
    {
      if(DebounceRight<BUTTON_DEBOUNCE_MAX) DebounceRight++;
      else if(DebounceRight==BUTTON_DEBOUNCE_MAX)
      {
        Set428RampMode(0, RM_VELOCITY);
        Write428Short(IDX_VMAX|MOTOR0, 2047);
        DebounceRight=0;

        i=Read428Short(IDX_VACTUAL|MOTOR0)+1;
        if(i>2047) i=2047;
        Write428Short(IDX_VTARGET|MOTOR0, i);
        Write428Short(IDX_VACTUAL|MOTOR0, i);
      }
    }
    else DebounceRight=0;
  }

  //LEDs "LEFT" und "RIGHT"
  i=Read428Short(IDX_VACTUAL);
  if(i>0)
  {
    LED_LEFT_OFF();
    LED_RIGHT_ON();
  }
  else if(i<0)
  {
    LED_LEFT_ON();
    LED_RIGHT_OFF();
  }
  else
  {
    LED_LEFT_OFF();
    LED_RIGHT_OFF();
  }

  //Taste "coolStep"
  if(BUTTON_COOL())
  {
    if(DebounceCool<BUTTON_DEBOUNCE_MAX) DebounceCool++;
    else if(DebounceCool==BUTTON_DEBOUNCE_MAX)
    {
      DebounceCool=BUTTON_DEBOUNCE_MAX+1;
      if(Get262SmartEnergyStallLevelMin(0)==0)
      {
        Set262SmartEnergyStallLevelMax(0, 3);
        Set262SmartEnergyUpStep(0, 3);
        Set262SmartEnergyDownStep(0, 0);
        Set262SmartEnergyIMin(0, 1);
        Set262StallGuardThreshold(0, 0);
        Set262StallGuardCurrentScale(0, 16);
        Set262SmartEnergyStallLevelMin(0, 2);
      }
      else
      {
        Set262SmartEnergyStallLevelMin(0, 0);
      }
    }
  }
  else DebounceCool=0;

  //LED "coolStep"
  if(Get262SmartEnergyStallLevelMin(0)>0)
    LED_COOL_ON();
  else
    LED_COOL_OFF();
}


//*** Hauptprogramm ***
int main(void)
{
  UINT t1;

  //Initialisierungen
  InitIO();
  InitADC();
  InitSysTimer();

  if(ExtSPIMode)
  {
  	AT91C_BASE_PIOB->PIO_CODR=BIT5|BIT4|BIT3;
  	ENABLE_DRIVERS();
  	for(;;)
  	{
  		RESET_WATCHDOG();
  	}
  }
  	
  InitSPI();

  InitUART(7);
  SetUARTTransmitDelay(0);

  InitUSB();

  //zustzliche Verzgerung vor der Initialisierung des Motortreibers
  RESET_WATCHDOG();
  t1=GetSysTimer();
  while(abs(GetSysTimer()-t1)<150);

  //Initialisierung der Rampengenerierung und des Motortreibers
  Init428();
  InitMotorDrivers();

  //Initialisierung des TMCL-Interpreters
  InitTMCL();

  ENABLE_DRIVERS();
  LED_LEFT_OFF();
  LED_RIGHT_OFF();
  LED_COOL_OFF();

  //Hauptschleife
  for(;;)
  {
    RESET_WATCHDOG();
    ProcessTMCL();
    CheckDrivers();
    ProcessButtons();
  }
}
