#include <stdlib.h>
#include "Eval429.h"
#include "Eval429-TMC428.h"
#include "Eval429-TMC262.h"


typedef struct
{
  USHORT V1;
  USHORT A1;
  UINT PMD_A1;
  USHORT V2;
  USHORT A2;
  UINT PMD_A2;
  USHORT V3;
  USHORT A3;
  UINT PMD_A3;
  USHORT D1;
  UINT PMD_D1;
} TRampParam;

typedef enum {RAMP_IDLE, RAMP_A1, RAMP_A2, RAMP_A3, RAMP_D} TRampState;

static TRampState RampState[N_O_MOTORS];
static TRampParam RampParam[N_O_MOTORS];
UCHAR RunCurrent[N_O_MOTORS];
static UCHAR BoostCurrent[N_O_MOTORS];


void SetRampParameter(UCHAR Type, UCHAR Motor, int Value)
{
  switch(Type)
  {
    case 0:
      RampParam[Motor].V1=Value;
      break;

    case 1:
      RampParam[Motor].A1=Value;
      SetAMax(Motor, Value);
      RampParam[Motor].PMD_A1=Read428Int(IDX_PMUL_PDIV|(Motor<<5));
      break;

    case 2:
      RampParam[Motor].V2=Value;
      break;

    case 3:
      RampParam[Motor].A2=Value;
      SetAMax(Motor, Value);
      RampParam[Motor].PMD_A2=Read428Int(IDX_PMUL_PDIV|(Motor<<5));
      break;

    case 4:
      RampParam[Motor].V3=Value;
      break;

    case 5:
      RampParam[Motor].A3=Value;
      SetAMax(Motor, Value);
      RampParam[Motor].PMD_A3=Read428Int(IDX_PMUL_PDIV|(Motor<<5));
      break;

    case 6:
      RampParam[Motor].D1=Value;
      SetAMax(Motor, Value);
      RampParam[Motor].PMD_D1=Read428Int(IDX_PMUL_PDIV|(Motor<<5));
      break;

    case 7:
      BoostCurrent[Motor]=Value;
      break;
  }
}


void CheckRamp(void)
{
  switch(RampState[0])
  {
    case RAMP_A1:
      if(abs(Read428Short(IDX_VACTUAL))>=RampParam[0].V1)
      {
        Write428Short(IDX_AMAX, RampParam[0].A2);
        Write428Int(IDX_PMUL_PDIV, RampParam[0].PMD_A2);
        Write428Short(IDX_VMAX, RampParam[0].V2);
        RampState[0]=RAMP_A2;
      }
      break;

    case RAMP_A2:
      if(abs(Read428Short(IDX_VACTUAL))>=RampParam[0].V2)
      {
        Write428Short(IDX_AMAX, RampParam[0].A3);
        Write428Int(IDX_PMUL_PDIV, RampParam[0].PMD_A3);
        Write428Short(IDX_VMAX, RampParam[0].V3);
        RampState[0]=RAMP_A3;
      }
      break;

    case RAMP_A3:
      if(abs(Read428Short(IDX_VACTUAL))>=RampParam[0].V3)
      {
        Set262StallGuardCurrentScale(0, RunCurrent[0]);
        Write428Short(IDX_AMAX, RampParam[0].D1);
        Write428Short(IDX_PMUL_PDIV, RampParam[0].PMD_D1);
        RampState[0]=RAMP_D;
      }
      break;

    case RAMP_D:
      if(abs(Read428Short(IDX_VACTUAL))<abs(Read428Short(IDX_VMAX)))
      {
        Set262StallGuardCurrentScale(0, BoostCurrent[0]);
        RampState[0]=RAMP_IDLE;
      }
      break;

    case RAMP_IDLE:
      break;
  }
}


void SpecialRamp(UCHAR Motor, int RelPos)
{
  int NewPosition;

  Set262StallGuardCurrentScale(0, BoostCurrent[0]);
  Write428Short(IDX_AMAX, RampParam[0].A1);
  Write428Int(IDX_PMUL_PDIV, RampParam[0].PMD_A1);
  Write428Short(IDX_VMAX, RampParam[0].V1);
  RampState[0]=RAMP_A1;

  NewPosition=Read428Int(Motor<<5|IDX_XACTUAL)+RelPos;
  Write428Int(Motor<<5|IDX_XTARGET, NewPosition);
  Set428RampMode(Motor, RM_RAMP);

  while(RampState[Motor]!=RAMP_IDLE && RampState[Motor]!=RAMP_D) CheckRamp();
}
