/****************************************************
  Projekt: TMC429+26x-Eval

  Modul:   Eval429-TMC424.c
           Funktionen fr den TMC424

  Datum:   7.3.2011 OK
*****************************************************/

#include "at91sam7x256.h"
#include "bits.h"
#include "Eval429.h"
#include "Eval429-TMC424.h"
#include "Eval429-SPI.h"


/*******************************************************************
   Funktion: ReadWrite423
   Parameter: Read: Array mit vier Bytes fr die gelesenen Daten
              Write: Array mit vier Bytes die in den TMC423
                     hineingeschrieben werden sollen.

   Rckgabewert: ---

   Zweck: SPI-Kommunikation mit dem TMC423
********************************************************************/
void ReadWrite423(UCHAR *Read, UCHAR *Write)
{
  Read[0]=ReadWriteSPI(SPI_DEV_TMC424, Write[0], FALSE);
  Read[1]=ReadWriteSPI(SPI_DEV_TMC424, Write[1], FALSE);
  Read[2]=ReadWriteSPI(SPI_DEV_TMC424, Write[2], FALSE);
  Read[3]=ReadWriteSPI(SPI_DEV_TMC424, Write[3], TRUE);
}


/*******************************************************************
   Funktion: Write423Bytes
   Parameter: Address: Registeraddresse des TMC423
              HiByte, MidByte, LoByte: Bytes, die in den TMC423
              hineingeschrieben werden sollen.

   Rckgabewert: ---

   Zweck: Beschreiben eines Registers des TMC423
********************************************************************/
void Write423Bytes(UCHAR Address, UCHAR HiByte, UCHAR MidByte, UCHAR LoByte)
{
  ReadWriteSPI(SPI_DEV_TMC424, Address|TMC423_WRITE, FALSE);
  ReadWriteSPI(SPI_DEV_TMC424, HiByte, FALSE);
  ReadWriteSPI(SPI_DEV_TMC424, MidByte, FALSE);
  ReadWriteSPI(SPI_DEV_TMC424, LoByte, TRUE);
}


/*******************************************************************
   Funktion: SetEncoderPrescaler
   Parameter: Which423: Auswahl des TMC423 bei Modulen mit mehr als 3 Achsen
              Index: Auswahl des Encoders (0, 1 oder 2)
              Prescaler: s. TMC423-Datenblatt
              SpecialFunctionBits: s. TMC423-Datenblatt

   Rckgabewert: ---

   Zweck: Setzen der Encoder-Prescaler-Werte und der Spezialfunktionen
          fr einen Encodereingang.
********************************************************************/
void SetEncoderPrescaler(UCHAR Index, UINT Prescaler, UCHAR SpecialFunctionBits)
{
  UCHAR RegAddr;
  UINT ps;
  UINT sf;

  switch(Index)
  {
    case 0: RegAddr=TMC423_ENC_CONF_1;
            break;
    case 1: RegAddr=TMC423_ENC_CONF_2;
            break;
    case 2: RegAddr=TMC423_ENC_CONF_3;
            break;
    default: return;
  }

  ps=Prescaler;
  ps<<=12;
  sf=SpecialFunctionBits;
  sf<<=7;
  ps|=sf;
  Write423Bytes(RegAddr, ps >> 16, ps >> 8, (UCHAR) ps);
}


/*******************************************************************
   Funktion: ReadEncoder
   Parameter: Index: Auswahl des Encoders (0, 1 oder 2)

   Rckgabewert: Wert des Positionszhlers

   Zweck: Auslesen der Position eines Encoders
********************************************************************/
int ReadEncoder(UCHAR Index)
{
  UCHAR Read423[4], Write423[4];
  UINT Position;

  switch(Index)
  {
    case 0: Write423[0]=TMC423_ENC_DATA_1;
            break;
    case 1: Write423[0]=TMC423_ENC_DATA_2;
            break;
    case 2: Write423[0]=TMC423_ENC_DATA_3;
            break;
    default: return 0;
  }

  ReadWrite423(Read423, Write423);
  Position=(Read423[1]<<16)|(Read423[2]<<8)|Read423[3];
  if(Read423[1] & 0x80) Position|=0xff000000;  //Vorzeichenbits erweitern

  return Position;
}


/*******************************************************************
   Funktion: WriteEncoder
   Parameter: Index: Auswahl des Encoders (0, 1 oder 2)
              Value: Neuer Wert fr den Positionszhler

   Rckgabewert: ---

   Zweck: Setzen des Positionszhlers eines Encoders auf einen neuen
          Wert.
********************************************************************/
void WriteEncoder(UCHAR Index, int Value)
{
  UCHAR RegAddr;

  switch(Index)
  {
    case 0: RegAddr=TMC423_ENC_DATA_1;
            break;
    case 1: RegAddr=TMC423_ENC_DATA_2;
            break;
    case 2: RegAddr=TMC423_ENC_DATA_3;
            break;
    default: return;
  }

  Write423Bytes(RegAddr, Value >> 16, Value >> 8, (UCHAR) Value);
}
