/****************************************************
  Projekt: TMCM-ARM (TMCL fr ARM-Module)

  Modul:   TMC428-ARM.c
           Funktionen fr den TMC428

  Datum:    2.4.2007 OK
*****************************************************/

#include "at91sam7x256.h"
#include "bits.h"

#include "Eval429.h"
#include "Eval429-SPI.h"
#include "Eval429-TMC428.h"
#include "Eval429-TMCL.h"
#include "Eval429-TMC262.h"


//Treibertabelle fr TMC262
const UCHAR DriverChainTable[64]=
{0x10, 0x10, 0x0e, 0x0d, 0x0c, 0x0b, 0x0a, 0x09, 0x08,
 0x10, 0x10, 0x06, 0x05, 0x04, 0x03, 0x02, 0x01, 0x00,
 0x10, 0x30,
 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11,
 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11, 0x11
};


//Funktionen

/*******************************************************************
   Funktion: ReadWrite428
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Read: Array mit vier Bytes fr die gelesenen Daten
              Write: Array mit vier Bytes die in den TMC428
                     hineingeschrieben werden sollen.

   Rckgabewert: ---

   Zweck: SPI-Kommunikation mit dem TMC428
********************************************************************/
void ReadWrite428(UCHAR *Read, UCHAR *Write)
{
  Read[0]=ReadWriteSPI(SPI_DEV_TMC429, Write[0], FALSE);
  Read[1]=ReadWriteSPI(SPI_DEV_TMC429, Write[1], FALSE);
  Read[2]=ReadWriteSPI(SPI_DEV_TMC429, Write[2], FALSE);
  Read[3]=ReadWriteSPI(SPI_DEV_TMC429, Write[3], TRUE);
}


/*******************************************************************
   Funktion: Write428Bytes
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Address: Registeradresse im TMC428
              Write: Array mit drei Bytes die in den TMC428
                     hineingeschrieben werden sollen.

   Rckgabewert: ---

   Zweck: Schreiben dreier Bytes in ein TMC428-Register.
********************************************************************/
void Write428Bytes(UCHAR Address, UCHAR *Bytes)
{
  UCHAR Write[4], Read[4];

  Write[0]=Address;
  Write[1]=Bytes[0];
  Write[2]=Bytes[1];
  Write[3]=Bytes[2];
  ReadWrite428(Read, Write);
}

/*******************************************************************
   Funktion: Write428Datagram
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Address: Registeradresse im TMC428
              HighByte: hchstwertiges Registerbyte
              MidbBte: mittleres Registerbyte
              LowbBte: niedrigstwertiges Registerbyte

   Rckgabewert: ---

   Zweck: Schreiben dreier Bytes in ein TMC428-Register.
********************************************************************/
void Write428Datagram(UCHAR Address, UCHAR HighByte, UCHAR MidByte, UCHAR LowByte)
{
  UCHAR Write[4], Read[4];

  Write[0]=Address;
  Write[1]=HighByte;
  Write[2]=MidByte;
  Write[3]=LowByte;
  ReadWrite428(Read, Write);
}

/*******************************************************************
   Funktion: Write428Zero
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Address: Registeradresse im TMC428

   Rckgabewert: ---

   Zweck: Ein Register im TMC428 auf Null setzen.
********************************************************************/
void Write428Zero(UCHAR Address)
{
  UCHAR Write[4], Read[4];

  Write[0]=Address;
  Write[1]=0;
  Write[2]=0;
  Write[3]=0;

  ReadWrite428(Read, Write);
}


/*******************************************************************
   Funktion: Write428Short
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Address: Registeradresse im TMC428
              Value: Zu schreibender Wert (16 Bit integer)

   Rckgabewert: ---

   Zweck: Schreiben eines 16-Bit-Wertes in ein TMC428-Register.
********************************************************************/
void Write428Short(UCHAR Address, int Value)
{
  UCHAR Write[4], Read[4];

  Write[0]=Address;
  Write[1]=0;
  Write[2]=Value >> 8;
  Write[3]=Value & 0xff;

  ReadWrite428(Read, Write);
}


/*******************************************************************
   Funktion: Write428Int
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Address: Registeradresse im TMC428
              Value: Zu schreibender Wert (24 Bit)

   Rckgabewert: ---

   Zweck: Schreiben eines 24-Bit-Wertes in ein TMC428-Register.
********************************************************************/
void Write428Int(UCHAR Address, int Value)
{
  UCHAR Write[4], Read[4];

  Write[0]=Address;
  Write[1]=Value >> 16;
  Write[2]=Value >> 8;
  Write[3]=Value & 0xff;

  ReadWrite428(Read, Write);
}


/*******************************************************************
   Funktion: Read428Status
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen

   Rckgabewert: Statusbyte des TMC428

   Zweck: Lesen des Statusbyte des TMC428
********************************************************************/
UCHAR Read428Status(void)
{
  return ReadWriteSPI(SPI_DEV_TMC429, 0x01, TRUE);
}


/*******************************************************************
   Funktion: Read428Bytes
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Address: Registeradresse des TMC428
              Bytes: Array fr die drei gelesenen Bytes

   Rckgabewert: Statusbyte des TMC428

   Zweck: Lesen eines TMC428-Registers als einzelne Bytes
********************************************************************/
UCHAR Read428Bytes(UCHAR Address, UCHAR *Bytes)
{
  UCHAR Read[4], Write[4];

  Write[0]=Address|TMC428_READ;
  ReadWrite428(Read, Write);

  Bytes[0]=Read[1];
  Bytes[1]=Read[2];
  Bytes[2]=Read[3];

  return Read[0];
}


/*******************************************************************
   Funktion: Read428Short
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Address: Registeradresse im TMC428

   Rckgabewert: gelesener Wert, Vorzeichenbits erweitert

   Zweck: Lesen eines 10- bzw. 11-Bit-Registers im TMC428 und
          Erweitern der Vorzeichenbits.
********************************************************************/
int Read428Short(UCHAR Address)
{
  UCHAR Read[4], Write[4];
  int Result;

  Write[0]=Address|TMC428_READ;
  ReadWrite428(Read, Write);

  Result=(Read[2]<<8)|(Read[3]);
  if(Result & 0x00000800) Result|=0xfffff000;

  return Result;
}


/*******************************************************************
   Funktion: Read428Int
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Address: Registeradresse im TMC428

   Rckgabewert: gelesener Wert, Vorzeichenbits erweitert

   Zweck: Lesen eines 24-Bit-Registers im TMC428 und
          Erweitern der Vorzeichenbits.
********************************************************************/
int Read428Int(UCHAR Address)
{
  UCHAR Read[4], Write[4];
  int Result;

  Write[0]=Address|TMC428_READ;
  ReadWrite428(Read, Write);

  Result=(Read[1]<<16)|(Read[2]<<8)|(Read[3]);
  if(Result & 0x00800000) Result|=0xff000000;

  return Result;
}


/*******************************************************************
   Funktion: Set428RampMode
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Axis: Motornummer im TMC428 (0, 1 oder 2)
              RampMode: Positionierungsmodus
                        (RM_RAMP/RM_SOFT/RM_VELOCITY/RM_HOLD)

   Rckgabewert: ---
   Zweck: Setzen des Positionierungsmodus des TMC428.
********************************************************************/
void Set428RampMode(UCHAR Axis, UCHAR RampMode)
{
  UCHAR Write[4], Read[4];

  Write[0] = Axis<<5|IDX_REFCONF_RM|TMC428_READ;
  ReadWrite428(Read, Write);

  Write[0] = Axis<<5|IDX_REFCONF_RM;
  Write[1] = Read[1];
  Write[2] = Read[2];
  Write[3] = RampMode;
  ReadWrite428(Read, Write);
}


/*******************************************************************
   Funktion: Set428SwitchMode
   Parameter: Which428: Auswahl des TMC428 bei Modulen mit mehr als 3 Achsen
              Axis: Motornummer im TMC428 (0, 1 oder 2)
              SwitchMode: Endschaltermodus des TMC428

   Rckgabewert: ---
   Zweck: Aktivieren bzw. Deaktivieren der Endschalter im TMC428.
********************************************************************/
void Set428SwitchMode(UCHAR Axis, UCHAR SwitchMode)
{
  UCHAR Write[4], Read[4];

  Write[0] = Axis<<5|IDX_REFCONF_RM|TMC428_READ;
  ReadWrite428(Read, Write);

  Write[0] = Axis<<5|IDX_REFCONF_RM;
  Write[1] = Read[1];
  Write[2] = SwitchMode;
  Write[3] = Read[3];
  ReadWrite428(Read, Write);
}


/*******************************************************************
   Funktion: SetAMax
   Parameter: Motor: Motornummer (0..N_O_MOTORS)
              AMax: Beschleunigung

   Rckgabewert: ---
   Zweck: Setzen der Beschleunigung und Berechnen von PMul/PDiv.
********************************************************************/
UCHAR SetAMax(UCHAR Motor, UINT AMax)
{
  int pdiv, pmul, pm, pd;
  float p, p_reduced;
  int ramp_div;
  int pulse_div;
  UCHAR PulseRampDiv;
  UCHAR Data[3];

  AMax&=0x000007ff;  //Begrenzen auf den Maximalwert des TMC428 (2047)
  Read428Bytes(IDX_PULSEDIV_RAMPDIV|Motor<<5, Data);
  PulseRampDiv=Data[1];
  pulse_div=PulseRampDiv>>4;
  ramp_div=PulseRampDiv & 0x0f;

  pm=-1; pd=-1; // -1 indicates : no valid pair found

  if(ramp_div>=pulse_div)
    p = AMax / ( 128.0 * (1<<(ramp_div-pulse_div)));  //Exponent positiv oder 0
  else
    p = AMax / ( 128.0 / (1<<(pulse_div-ramp_div)));  //Exponent negativ

  p_reduced = p*0.988;

  for (pdiv=0; pdiv<=13; pdiv++)
  {
    pmul = (int)(p_reduced * 8.0 * (1<<pdiv)) - 128;

    if ( (0 <= pmul) && (pmul <= 127) )
    {
      pm = pmul + 128;
      pd = pdiv;
    }
  }

  Data[0]=0;
  Data[1]=(UCHAR) pm;
  Data[2]=(UCHAR) pd;
  Write428Bytes((Motor<<5)|IDX_PMUL_PDIV, Data);
  Write428Short((Motor<<5)|IDX_AMAX, AMax);

  return 0;
}


/*******************************************************************
   Funktion: Init428
   Parameter: ---

   Rckgabewert: ---
   Zweck: Initialisierung des/der TMC428.
********************************************************************/
void Init428(void)
{
  UINT Address;
  UCHAR Motor;

  for(Motor=0; Motor<3; Motor++)
  {
    for(Address=0; Address<=IDX_XLATCHED; Address++)
      Write428Zero(Address|(Motor<<5));
  }

  for(Address=128; Address<192; Address++)
    Write428Datagram(Address, 0, DriverChainTable[Address-127], DriverChainTable[Address-128]);

  Write428Datagram(IDX_SMGP, 0x01, 0x00, 0x02);
  Write428Int(IDX_IF_CONFIG_429, IFCONF_EN_SD|IFCONF_EN_REFR|IFCONF_INV_REF|IFCONF_SDO_INT);

  Write428Datagram(IDX_PULSEDIV_RAMPDIV|MOTOR0, 0x00, 0x37, 0x04);
  Write428Datagram(IDX_PULSEDIV_RAMPDIV|MOTOR1, 0x00, 0x37, 0x04);
  Write428Datagram(IDX_PULSEDIV_RAMPDIV|MOTOR2, 0x00, 0x37, 0x04);

  Write428Int(IDX_VMIN|MOTOR0, 1);
  Write428Int(IDX_VMIN|MOTOR1, 1);
  Write428Int(IDX_VMIN|MOTOR2, 1);

  Write428Int(IDX_VMAX|MOTOR0, 1000);
  Write428Int(IDX_VMAX|MOTOR1, 1000);
  Write428Int(IDX_VMAX|MOTOR2, 1000);

  SetAMax(0, 50);
  SetAMax(1, 50);
  SetAMax(2, 50);

  Set428SwitchMode(0, 0);    //Endschalter fr Motor 0 sind aktiv
  Set428SwitchMode(1, NO_REF);
  Set428SwitchMode(2, NO_REF);
}


/*******************************************************************
   Funktion: HardStop
   Parameter: Motor: Motorindex (0..N_O_MOTORS)

   Rckgabewert: ---
   Zweck: Schnellstop des angegebenen Motors.
********************************************************************/
void HardStop(UINT Motor)
{
  Set428RampMode(Motor, RM_VELOCITY);
  Write428Zero(Motor<<5|IDX_VTARGET);
  Write428Zero(Motor<<5|IDX_VACTUAL);
}


void SelectSPIMode(void)
{
  Set262DriverReadSelect(0, TMC262_RB_SMART_ENERGY);
  Set262DriverStepDirectionOff(0, TRUE);
  Set262DriverStepDirectionOff(1, TRUE);
  Set262DriverStepDirectionOff(2, TRUE);
  Write428Int(IDX_IF_CONFIG_429, IFCONF_EN_REFR|IFCONF_INV_REF|IFCONF_SDO_INT);
  Write428Datagram(IDX_COVER_POS_LEN, 0, 0, 20);  //20 Bits Cover-Telegramm
  Write428Datagram(IDX_SMGP, 0x01, 0x07, 0x00);
  DRIVE_MODE_SPI();
}


void SelectStepDirMode(void)
{
  Write428Datagram(IDX_SMGP, 0x01, 0x00, 0x02);
  Write428Int(IDX_IF_CONFIG_429, IFCONF_EN_SD|IFCONF_EN_REFR|IFCONF_INV_REF|IFCONF_SDO_INT);
  DRIVE_MODE_SD();
  Set262DriverStepDirectionOff(0, FALSE);
  Set262DriverStepDirectionOff(1, FALSE);
  Set262DriverStepDirectionOff(2, FALSE);
}
